"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HtmlWebpackInjectExternalsPlugin = void 0;
/* eslint-disable import/first */
const fs_1 = require("fs");
const path_1 = require("path");
const chalk_1 = require("chalk");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const url_join_1 = __importDefault(require("url-join"));
const mergeExternals_1 = require("./mergeExternals");
class HtmlWebpackInjectExternalsPlugin {
    constructor(options) {
        this.name = 'HtmlWebpackInjectExternalsPlugin';
        this.options = options;
    }
    apply(compiler) {
        if (this.options.externals) {
            const externals = mergeExternals_1.mergeExternals(this.options.externals, compiler.options.externals);
            if (externals) {
                compiler.options.externals = externals;
            }
        }
        const generalHost = this.options.host;
        const getUmdPath = (pkg) => {
            const { host, path, fullPath, name } = pkg;
            if (fullPath) {
                return {
                    url: fullPath,
                    attributes: pkg.attributes,
                };
            }
            let pkgInfo = {};
            /** mono packages project need to inspact node_modules in more place */
            try {
                // eslint-disable-next-line @typescript-eslint/no-var-requires,global-require,import/no-dynamic-require
                pkgInfo = require(`${name}/package.json`);
            }
            catch (e) {
                const currentPathPkgPath = path_1.join(process.cwd(), 'node_modules', `${name}/package.json`);
                if (fs_1.existsSync(currentPathPkgPath)) {
                    // eslint-disable-next-line @typescript-eslint/no-var-requires,global-require,import/no-dynamic-require
                    pkgInfo = require(currentPathPkgPath);
                }
                else {
                    throw new Error(`package ${name} missing umd path`);
                }
            }
            const { version, unpkg, browser, umd } = pkgInfo;
            const unpkgHost = host || generalHost;
            /**
             * try to guess the browser env usage file path
             * the guess order is reference from unpkg project
             * */
            const pkgUmdPath = path !== undefined ? path : unpkg || browser || umd || '';
            if (!unpkgHost) {
                throw new Error(`package ${name} missing host`);
            }
            const url = fullPath || url_join_1.default(unpkgHost, `${name}@${version}`, pkgUmdPath);
            return {
                url,
                version,
                attributes: pkg.attributes,
            };
        };
        const deps = this.options.packages.map(getUmdPath);
        const tags = deps.map(d => {
            const result = d.url.endsWith('.css')
                ? {
                    tagName: 'link',
                    voidTag: true,
                    attributes: Object.assign(Object.assign({ rel: 'stylesheet', href: d.url }, this.options.attributes), d.attributes),
                }
                : {
                    tagName: 'script',
                    voidTag: false,
                    attributes: Object.assign(Object.assign({ defer: false, src: d.url }, this.options.attributes), d.attributes),
                };
            return result;
        });
        compiler.hooks.compilation.tap(this.name, compilation => {
            HtmlWebpackPlugin.getHooks(compilation).alterAssetTagGroups.tap(this.name, data => {
                /** remove repeat */
                const toPrependTags = [];
                tags.reverse().forEach(tag => {
                    const url = tag.tagName === 'script' ? tag.attributes.src : tag.attributes.href;
                    const exist = data.headTags.some(headTag => {
                        const headTagUrl = headTag.tagName === 'script' ? headTag.attributes.src : headTag.attributes.href;
                        return headTagUrl === url;
                    });
                    if (!exist) {
                        data.headTags.unshift(tag);
                        toPrependTags.push(tag);
                    }
                });
                console.log(chalk_1.rgb(73, 204, 144)(`[${this.name}] has prepend these resources in your html head:`));
                toPrependTags.forEach((tag) => {
                    console.log(chalk_1.rgb(137, 191, 4)(JSON.stringify(tag)));
                });
                return data;
            });
        });
    }
}
exports.HtmlWebpackInjectExternalsPlugin = HtmlWebpackInjectExternalsPlugin;
